#include "llvm/Pass.h"
#include "llvm/IR/Module.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/MDBuilder.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/Support/CommandLine.h"

using namespace llvm;

// 添加命令行选项，用于指定项目名称
static cl::opt<std::string> ProjectName(
    "project-name",
    cl::desc("Specify the project name to be added as metadata"),
    cl::value_desc("name"),
    cl::Required // 设置为必需参数
);

namespace {
struct ProjectMetadataPass : public ModulePass {
    static char ID;
    ProjectMetadataPass() : ModulePass(ID) {}

    bool runOnModule(Module &M) override {
        bool Modified = false;
        LLVMContext &Ctx = M.getContext();

        // 创建项目名称的metadata字符串
        MDString *ProjectStr = MDString::get(Ctx, ProjectName.getValue());
        
        // 为每个非声明函数添加metadata
        for (Function &F : M) {
            if (F.isDeclaration())
                continue;

            // 检查函数是否已经有项目metadata
            if (F.getMetadata("project_source")) {
                errs() << "Warning: Function " << F.getName() 
                       << " already has project metadata.\n";
                continue;
            }

            // 创建metadata节点
            MDNode *ProjectMD = MDNode::get(Ctx, ProjectStr);
            
            // 为函数添加metadata
            F.setMetadata("project_source", ProjectMD);
            
            errs() << "Added project metadata '" << ProjectName 
                   << "' to function: " << F.getName() << "\n";
            
            Modified = true;
        }

        return Modified;
    }
};
}

char ProjectMetadataPass::ID = 0;
static RegisterPass<ProjectMetadataPass> 
    X("add-project-metadata", "Add project source metadata to functions");